// Uncomment line below if you want to use rectangles
// #define VOT_RECTANGLE
#define VOT_POLYGON
#include <numeric>
#include <opencv2/core/utility.hpp>
#include <opencv2/core/core.hpp>
#include <opencv2/tracking.hpp>
#include <opencv2/videoio.hpp>
#include <opencv2/highgui.hpp>
#include <iostream>
#include <cstring>
#include "samples_utility.hpp"

#include "vot.h"

using namespace std;
using namespace cv;
using namespace trax;

#include <chrono>

int main(int argc, char* argv[])
{
    // Initialize the communcation
    VOT vot; 
    VOTRegion region = vot.region();
    // get the path to the first frame
    string path = vot.frame();
    cv::Mat image = cv::imread(path);
    Ptr<TrackerCSRT> tracker = TrackerCSRT::create();
    if(argc > 1) {
    	FileStorage fs(argv[1], FileStorage::READ);
    	tracker->read( fs.root());
    }
    // get groundtruth region
    cv::Rect rect;
    rect << region;

#ifdef VOT_POLYGON
    // Mat mask = Mat::zeros(frame.height(), frame.width(), CV_32FC1);
    Mat mask = Mat::zeros(rect.size(), CV_32FC1);
    int n = region.count();
    //Point poly_points[1][n];
    std::vector<cv::Point> poly_points(n);
    std::vector<float> elements;
    //Translate x and y to rects start position
    int sx = rect.x;
    int sy = rect.y;
    for (int i = 0; i < n; ++i) {
        poly_points[i] = Point( round(region.get_x(i)) - sx, round(region.get_y(i) - sy) );
        elements.push_back(round(region.get_x(i)));
        elements.push_back(round(region.get_y(i)));
    }
    float xMin = std::min(elements[0], std::min(elements[2], std::min(elements[4], elements[6])));
    float yMin = std::min(elements[1], std::min(elements[3], std::min(elements[5], elements[7])));
    float xMax = std::max(elements[0], std::max(elements[2], std::max(elements[4], elements[6])));
    float yMax = std::max(elements[1], std::max(elements[3], std::max(elements[5], elements[7])));

    // approximate polygon with axis-aligned bounding box
    // keep approximately the same area
    float cx = (elements[0] + elements[2] + elements[4] + elements[6]) / 4.f;
    float cy = (elements[1] + elements[3] + elements[5] + elements[7]) / 4.f;
    float A1 = std::sqrt(std::pow(elements[0] - elements[2], 2) + std::pow(elements[1] - elements[3], 2)) * 
        std::sqrt(std::pow(elements[2] - elements[4], 2) + std::pow(elements[3] - elements[5], 2));
    float A2 = (xMax - xMin) * (yMax - yMin);
    float s = std::sqrt(A1 / A2);
    float w = s * (xMax - xMin);
    float h = s * (yMax - yMin);
    float x0 = cx - w / 2;
    float y0 = cy - h / 2;

    // tracker->setInitialScale(s);
    cv::fillConvexPoly(mask, poly_points, Scalar(1.0), CV_AA);
    mask.convertTo(mask, CV_32FC1);

    tracker->setInitialMask(mask);
#endif 
    // Rect      roi = cv::Rect(x0, y0, w, h);
    tracker->init(image, rect);
    // tracker->init(image, roi);

    // tracking loop
    while (true) {
        // Get the next frame
        path = vot.frame();
        // Are we done?
        if (path.empty()) 
            break;
        image = cv::imread(path);
        // get region from the tracker
        Rect2d roi;
        tracker->update(image,roi);
        Rect rect = roi;
        // Report the position of the tracker
        vot.report(rect);
    }

    // Finishing the communication is completed automatically with the destruction
    // of the communication object (if you are using pointers you have to explicitly
    // delete the object).

    return 0;
}
